__doc__ = "寻路算法模块"


from pathfinding.finder.finder import Finder

from logis.math import *

from ._a_star import AStarPathFinding
from .base import *
from .model import *

# from ._dijkstra import DijkstraPathFinding


if __name__ == "__main__":
    pass
